#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import os
import uuid
from urllib.parse import urlparse

from opendatalab.__version__ import __version__

UUID = str(uuid.uuid3(uuid.NAMESPACE_URL, f"opendatalab-python-sdk/{__version__}")).replace("-", "")


def parse_url(url: str):
    o = urlparse(url)
    host = f"{o.scheme}://{o.hostname}"
    if o.port is not None:
        host += ":" + str(o.port)

    dataset_name = o.path.split("/")[-1]
    return host, dataset_name


def get_api_token_from_env():
    return os.environ.get("OPENDATALAB-API-TOKEN", "")

SYMBOLS = {
    'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
    'customary_ext': ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
                      'zetta', 'iotta'),
    'iec': ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
    'iec_ext': ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
                'zebi', 'yobi'),
}


def bytes2human(n, format='%(value).1f%(symbol)s', symbols='customary'):
    """
    Convert n bytes into a human-readable string based on format.
    symbols can be either "customary", "customary_ext", "iec" or "iec_ext",
    see: http://goo.gl/kTQMs
    """

    n = int(n)
    if n < 0:
        raise ValueError("n < 0")
    symbols = SYMBOLS[symbols]
    prefix = {}
    for i, s in enumerate(symbols[1:]):
        prefix[s] = 1 << (i + 1) * 10
    for symbol in reversed(symbols[1:]):
        if n >= prefix[symbol]:
            value = float(n) / prefix[symbol]
            return format % locals()
    return format % dict(symbol=symbols[0], value=n)


def human2bytes(s):
    """
    Attempts to guess the string format based on default symbols
    set and return the corresponding bytes as an integer.
    When unable to recognize the format ValueError is raised.
    """
    init = s
    num = ""
    while s and s[0:1].isdigit() or s[0:1] == '.':
        num += s[0]
        s = s[1:]
    num = float(num)
    letter = s.strip()
    for name, sset in SYMBOLS.items():
        if letter in sset:
            break
    else:
        if letter == 'k':
            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
            sset = SYMBOLS['customary']
            letter = letter.upper()
        else:
            raise ValueError("can't interpret %r" % init)
    prefix = {sset[0]: 1}
    for i, s in enumerate(sset[1:]):
        prefix[s] = 1 << (i + 1) * 10
    return int(num * prefix[letter])




#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
# from datetime import date


"""OpenDataLab python SDK version info."""

__url__ = "https://opendatalab.org.cn"
__version__ = "0.0.4"
__svc__ = '2.0'
odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"


#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
"""Customization of the click classes for OpenDataLab CLI."""

from typing import Any
import click


class CustomCommand(click.Command):
    """Wrapper class of ``click.Command`` for CLI commands with custom help.

    Arguments:
        kwargs: The keyword arguments pass to ``click.Command``.

    """

    def __init__(self, **kwargs: Any) -> None:
        self.synopsis = kwargs.pop("synopsis", ())
        super().__init__(**kwargs)

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        """Writes the custom help into the formatter if it exists.

        Override the original ``click.Command.format_help`` method by
        adding :meth:`CustomCommand.format_synopsis` to form custom help message.

        Arguments:
            ctx: The context of the command.
            formatter: The help formatter of the command.

        """
        formatter.width = 100
        self.format_usage(ctx, formatter)
        self.format_help_text(ctx, formatter)
        self.format_synopsis(formatter)
        self.format_options(ctx, formatter)
        self.format_epilog(ctx, formatter)

    def format_synopsis(self, formatter: click.HelpFormatter) -> None:
        """Write the synopsis to the formatter if exists.

        Arguments:
            formatter: The help formatter of the command.
        """
        if not self.synopsis:
            return

        with formatter.section("Synopsis"):
            for example in self.synopsis:
                formatter.write_text(example)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
from rich import box
from rich.console import Console
from rich.table import Table

from opendatalab.cli.utility import ContextInfo, exception_handler
from opendatalab.utils import bytes2human


def _format_types(info_data, type_name):
    types_str = ""
    if type_name in info_data['attrs'].keys():
        types_list = info_data['attrs'][type_name]
        if types_list and len(types_list) > 0:
            types_str = ", ".join([x['name']['en'] for x in types_list])

    return types_str


def reformat_info_data(info_data):
    license_str = _format_types(info_data, 'license')
    publisher_str = _format_types(info_data, 'publisher')
    media_types_str = _format_types(info_data, 'mediaTypes')
    label_types_str = _format_types(info_data, 'labelTypes')
    task_types_str = _format_types(info_data, 'taskTypes')
    tags_str = _format_types(info_data, 'tags')

    data_introduction = info_data['introduction']['en']
    introduction_str = ""
    if data_introduction and len(data_introduction) > 0:
        introduction_str = data_introduction[:97] + '...'

    citation_data = info_data['attrs']['citation']
    citation_str = ""
    if citation_data and len(citation_data) > 0:
        citation_str = citation_data.strip("```").replace('\r', '').replace('\n', '')

    similar_data_list = info_data['similar']
    similar_ds_str = ""
    if similar_data_list and len(similar_data_list) > 0:
        similar_ds_str = ", ".join([x['name'] for x in similar_data_list])

    info_data_result = {
        'Name': info_data['name'],
        'File Bytes': str(bytes2human(info_data['attrs']['fileBytes'])),
        'File Count': str(info_data['attrs']['fileCount']),
        'Introduction': introduction_str,
        'Issue Time': info_data['attrs']['publishDate'],
        'License': license_str,
        'Author': publisher_str,
        'Data Type': media_types_str,
        'Label Type': label_types_str,
        'Task Type': task_types_str,
        'Tags': tags_str,
        'HomePage': info_data['attrs']['publishUrl'],
        'Citation': citation_str,
        'Similar Datasets': similar_ds_str,
    }

    return info_data_result


@exception_handler
def implement_info(obj: ContextInfo, dataset: str) -> None:
    """
    implement for displaying dataset info
    Args:
        obj (ContextInfo): context object
        dataset (str): dataset name

    Returns:

    """
    client = obj.get_client()
    odl_api = client.get_api()
    info_data = odl_api.get_info(dataset)

    info_data_result = reformat_info_data(info_data)
    console = Console()
    table = Table(show_header=True, header_style='bold cyan', box=box.ASCII2)
    table.add_column("Field", width=20, justify='full', overflow='fold')
    table.add_column("Content", width=120, justify='full', overflow='fold')

    for key in info_data_result.keys():
        val = info_data_result[key]
        val = "" if not val else val
        table.add_row(key, val, end_section=True)

    console.print(table)



#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import sys
import operator
import click

from opendatalab.__version__ import __version__
from opendatalab.cli.utility import ContextInfo, exception_handler


@exception_handler
def implement_upgrade(obj: ContextInfo):
    """
        check cli version compare with svc
    Args:
        obj (ContextInfo):

    Returns:

    """
    client = obj.get_client()
    odl_api = client.get_api()
    version_info = odl_api.check_version()

    latest_major_version = version_info['majorVersion']
    latest_minor_version = version_info['minorVersion']
    latest_service_version = version_info['serviceVersion']
    is_beta_latest = version_info['isBeta']

    # to discuss: if beta False, ignore beta version, set beta_version = 0
    if is_beta_latest:
        latest_beta_version = version_info['betaVersion']
        latest_res_version = latest_major_version + '.' + latest_minor_version + 'b' + str(latest_beta_version)
    else:
        latest_beta_version = 0
        latest_res_version = latest_major_version + '.' + latest_minor_version + '.' + str(latest_beta_version)

    # installed_svc = __svc__
    installed_version = __version__
    is_installed_beta = installed_version.find("b") != -1

    if is_installed_beta:
        cur_maj_min_ver = installed_version.split('b')[0]
        cur_major_ver = cur_maj_min_ver[:cur_maj_min_ver.rfind('.')]
        cur_minor_ver = cur_maj_min_ver[cur_maj_min_ver.rfind('.') + 1:]
        cur_b_ver = installed_version.split('b')[1]
    else:
        cur_major_ver = installed_version[:installed_version.rfind('.')]
        cur_minor_ver = installed_version[installed_version.rfind('.') + 1:]
        cur_b_ver = 0
        pass

    """
    version :
        released: 0.0.1.0 -> 0.0.2.0 ... -> 0.1.0.0
        beta    : 0.0.1b10 -> 0.0.1b11 .. -> 0.0.1b90...
        major_version: 0.0
        minor_version: 1
        is_beta: bool, True->b, False->''
        beta_version: released -> 0, beta -> 90
    upgrade flag
        -1: pre-check, 
        0: checked-no-need-grade, 
        1: checked-need-upgrade
    """
    # check_ret_code = -1

    if operator.gt(float(latest_major_version), float(cur_major_ver)):
        check_ret_code = 1
        click.secho(f"check version: installed :{installed_version}, latest: {latest_res_version}", fg='green')
        click.secho(f"[Fatal]: upgrade needed, please use 'pip install -U opendatalab'", fg='red')
        obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)
        sys.exit(-1)

    elif operator.lt(float(latest_major_version), float(cur_major_ver)):
        check_ret_code = 0
        obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)

    else:
        if operator.gt(float(latest_minor_version), float(cur_minor_ver)):
            check_ret_code = 1
            click.secho(f"odl version: current: {installed_version}, latest: {latest_res_version}", fg='green')
            click.secho(f"[Fatal]: upgrade needed, please use 'pip install -U opendatalab'", fg='red')
            obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)
            sys.exit(-1)

        elif operator.lt(float(latest_minor_version), float(cur_minor_ver)):
            check_ret_code = 0
            obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)

        else:
            if is_installed_beta and is_beta_latest:
                if operator.gt(float(latest_beta_version), float(cur_b_ver)):
                    check_ret_code = 1
                    click.secho(f"odl version: current: {installed_version}, latest: {latest_res_version}", fg='green')
                    click.secho(f"[Fatal]: upgrade needed, please use 'pip install -U opendatalab'", fg='red')
                    obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)
                    sys.exit(-1)
                else:
                    check_ret_code = 0
                    obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)

            elif is_beta_latest and not is_installed_beta:
                # TODO: keep latest released or beta version
                check_ret_code = 1
                click.secho(f"odl version: current: {installed_version}, latest: {latest_res_version}", fg='green')
                click.secho(f"[Fatal]: upgrade needed, please use 'pip install -U opendatalab'", fg='red')
                obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)
                sys.exit(-1)

            else:
                check_ret_code = 0
                obj.set_check_info(latest_version=latest_res_version, check_ret=check_ret_code)